@@ -1,9 +1,9 @@
 /*
- * "$Id: aix.c 670 2006-04-05 15:08:08Z mike $"
+ * "$Id: aix.c 749 2007-01-14 20:55:33Z mike $"
  *
  *   AIX package gateway for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2006 by Easy Software Products.
+ *   Copyright 1999-2007 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -136,10 +136,14 @@ make_aix(const char     *prodname,	/* I - Product short name */
     return (1);
   }
 
+  fprintf(fp, "4 R I %s {\n", prodname);
+
   aix_fileset(fp, prodname, dist, NULL);
   for (i = 0; i < dist->num_subpackages; i ++)
     aix_fileset(fp, prodname, dist, dist->subpackages[i]);
 
+  fputs("}\n", fp);
+
   fclose(fp);
 
  /*
@@ -409,14 +413,27 @@ aix_fileset(FILE       *fp,		/* I - File to write to */
   * Start fileset definition...
   */
 
-  fprintf(fp, "4 R I %s {\n", prodname);
-
   if (subpackage)
     fprintf(fp, "%s.%s", prodname, subpackage);
   else
     fprintf(fp, "%s", prodname);
 
-  fprintf(fp, " %s 01 N B x %s\n", aix_version(dist->version), dist->product);
+  fprintf(fp, " %s 01 N B x ", aix_version(dist->version));
+
+  if (subpackage)
+  {
+    for (i = 0; i < dist->num_descriptions; i ++)
+    {
+      if (dist->descriptions[i].subpackage == subpackage)
+      {
+	fprintf(fp, "%s\n", dist->descriptions[i].description);
+	break;
+      }
+    }
+  }
+  else
+    fprintf(fp, "%s\n", dist->product);
+
 
  /*
   * Dependencies...
@@ -464,7 +481,6 @@ aix_fileset(FILE       *fp,		/* I - File to write to */
 
   fputs("%\n", fp);
   fputs("]\n", fp);
-  fputs("}\n", fp);
 }
 
 
@@ -864,7 +880,19 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
   */
 
   snprintf(filename, sizeof(filename), "%s/lpp.README", directory);
-  copy_file(filename, dist->license, 0644, 0, 0);
+
+  if (dist->license[0])
+    copy_file(filename, dist->license, 0644, 0, 0);
+  else if (dist->readme[0])
+    copy_file(filename, dist->readme, 0644, 0, 0);
+  else if ((fp = fopen(filename, "w")) != NULL)
+    fclose(fp);
+  else
+  {
+    fprintf(stderr, "epm: Unable to create .README file \"%s\" - %s\n",
+            filename, strerror(errno));
+    return (1);
+  }
 
  /*
   * Create the liblpp.a file...
@@ -892,8 +920,9 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
              prodname, prodname);
   }
 
-  if (run_command(directory, "ar rc %s lpp.README", filename))
-    return (1);
+  if (!subpackage)
+    if (run_command(directory, "ar rc %s lpp.README", filename))
+      return (1);
 
   for (i = 0; i < (sizeof(files) / sizeof(files[0])); i ++)
   {
@@ -904,7 +933,7 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
       continue;
 
     if (run_command(directory, "ar rc %s %s.%s",
-                    filename, prodname, files[i]))
+                    filename, prodfull, files[i]))
       return (1);
   }
 
@@ -913,5 +942,5 @@ write_liblpp(const char     *prodname,	/* I - Product short name */
 
 
 /*
- * End of "$Id: aix.c 670 2006-04-05 15:08:08Z mike $".
+ * End of "$Id: aix.c 749 2007-01-14 20:55:33Z mike $".
  */
