@@ -1,9 +1,9 @@
 /*
- * "$Id: bsd.c,v 1.9 2002/10/17 18:13:13 mike Exp $"
+ * "$Id: bsd.c 670 2006-04-05 15:08:08Z mike $"
  *
- *   FreeBSD package gateway for the ESP Package Manager (EPM).
+ *   Free/Net/OpenBSD package gateway for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2002 by Easy Software Products.
+ *   Copyright 1999-2006 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
  *
  * Contents:
  *
- *   make_bsd() - Make a FreeBSD software distribution package.
+ *   make_bsd()        - Make a Free/Net/OpenBSD software distribution package.
+ *   make_subpackage() - Create a subpackage...
  */
 
 /*
@@ -28,7 +29,16 @@
 
 
 /*
- * 'make_bsd()' - Make a FreeBSD software distribution package.
+ * Local functions...
+ */
+
+static int	make_subpackage(const char *prodname, const char *directory,
+		                const char *platname, dist_t *dist,
+			        const char *subpackage);
+
+
+/*
+ * 'make_bsd()' - Make a Free/Net/OpenBSD software distribution package.
  */
 
 int					/* O - 0 = success, 1 = fail */
@@ -39,7 +49,38 @@ make_bsd(const char     *prodname,	/* I - Product short name */
 	 struct utsname *platform)	/* I - Platform information */
 {
   int		i;			/* Looping var */
+
+
+  if (make_subpackage(prodname, directory, platname, dist, NULL))
+    return (1);
+
+  for (i = 0; i < dist->num_subpackages; i ++)
+    if (make_subpackage(prodname, directory, platname, dist,
+                        dist->subpackages[i]))
+      return (1);
+
+  return (0);
+}
+
+
+/*
+ * 'make_subpackage()' - Create a subpackage...
+ */
+
+static int				/* O - 0 = success, 1 = fail */
+make_subpackage(const char     *prodname,
+					/* I - Product short name */
+        	const char     *directory,
+					/* I - Directory for distribution files */
+        	const char     *platname,
+					/* I - Platform name */
+		dist_t         *dist,	/* I - Distribution information */
+		const char     *subpackage)
+					/* I - Subpackage name */
+{
+  int		i;			/* Looping var */
   FILE		*fp;			/* Spec file */
+  char		prodfull[1024];		/* Full subpackage name */
   char		name[1024];		/* Full product name */
   char		commentname[1024];	/* pkg comment filename */
   char		descrname[1024];	/* pkg descr filename */
@@ -56,34 +97,38 @@ make_bsd(const char     *prodname,	/* I - Product short name */
   char		current[1024];		/* Current directory */
 
 
-  REF(platform);
-
-  if (Verbosity)
-    puts("Creating FreeBSD pkg distribution...");
-
   getcwd(current, sizeof(current));
 
-  if (dist->relnumber)
+  if (subpackage)
+    snprintf(prodfull, sizeof(prodfull), "%s-%s", prodname, subpackage);
+  else
+    strlcpy(prodfull, prodname, sizeof(prodfull));
+
+  if (Verbosity)
+    printf("Creating %s *BSD pkg distribution...\n", prodfull);
+
+  if (dist->release[0])
   {
     if (platname[0])
-      snprintf(name, sizeof(name), "%s-%s-%d-%s", prodname, dist->version, dist->relnumber,
-              platname);
+      snprintf(name, sizeof(name), "%s-%s-%s-%s", prodfull, dist->version,
+               dist->release, platname);
     else
-      snprintf(name, sizeof(name), "%s-%s-%d", prodname, dist->version, dist->relnumber);
+      snprintf(name, sizeof(name), "%s-%s-%s", prodfull, dist->version,
+               dist->release);
   }
   else if (platname[0])
-    snprintf(name, sizeof(name), "%s-%s-%s", prodname, dist->version, platname);
+    snprintf(name, sizeof(name), "%s-%s-%s", prodfull, dist->version, platname);
   else
-    snprintf(name, sizeof(name), "%s-%s", prodname, dist->version);
+    snprintf(name, sizeof(name), "%s-%s", prodfull, dist->version);
 
  /*
   * Write the descr file for pkg...
   */
 
   if (Verbosity)
-    puts("Creating descr file...");
+    printf("Creating %s.descr file...\n", prodfull);
 
-  snprintf(descrname, sizeof(descrname), "%s/%s.descr", directory, prodname);
+  snprintf(descrname, sizeof(descrname), "%s/%s.descr", directory, prodfull);
 
   if ((fp = fopen(descrname, "w")) == NULL)
   {
@@ -101,9 +146,10 @@ make_bsd(const char     *prodname,	/* I - Product short name */
   */
 
   if (Verbosity)
-    puts("Creating comment file...");
+    printf("Creating %s.comment file...\n", prodfull);
 
-  snprintf(commentname, sizeof(commentname), "%s/%s.comment", directory, prodname);
+  snprintf(commentname, sizeof(commentname), "%s/%s.comment", directory,
+           prodfull);
 
   if ((fp = fopen(commentname, "w")) == NULL)
   {
@@ -113,18 +159,19 @@ make_bsd(const char     *prodname,	/* I - Product short name */
   }
 
   fprintf(fp, "Summary: %s\n", dist->product);
-  fprintf(fp, "Name: %s\n", prodname);
+  fprintf(fp, "Name: %s\n", prodfull);
   fprintf(fp, "Version: %s\n", dist->version);
-  fprintf(fp, "Release: %d\n", dist->relnumber);
+  fprintf(fp, "Release: %s\n", dist->release);
   fprintf(fp, "Copyright: %s\n", dist->copyright);
   fprintf(fp, "Packager: %s\n", dist->packager);
   fprintf(fp, "Vendor: %s\n", dist->vendor);
-  fprintf(fp, "BuildRoot: %s/%s/buildroot\n", current, directory);
+  fprintf(fp, "BuildRoot: %s/%s/%s.buildroot\n", current, directory, prodfull);
   fputs("Group: Applications\n", fp);
 
   fputs("Description:\n\n", fp);
   for (i = 0; i < dist->num_descriptions; i ++)
-    fprintf(fp, "%s\n", dist->descriptions[i]);
+    if (dist->descriptions[i].subpackage == subpackage)
+      fprintf(fp, "%s\n", dist->descriptions[i].description);
 
   fclose(fp);
 
@@ -133,9 +180,9 @@ make_bsd(const char     *prodname,	/* I - Product short name */
   */
 
   if (Verbosity)
-    puts("Creating plist file...");
+    printf("Creating %s.plist file...\n", prodfull);
 
-  snprintf(plistname, sizeof(plistname), "%s/%s.plist", directory, prodname);
+  snprintf(plistname, sizeof(plistname), "%s/%s.plist", directory, prodfull);
 
   if ((fp = fopen(plistname, "w")) == NULL)
   {
@@ -144,22 +191,41 @@ make_bsd(const char     *prodname,	/* I - Product short name */
     return (1);
   }
 
-  fprintf(fp, "@srcdir %s/%s/buildroot\n", current, directory);
+ /*
+  * FreeBSD and NetBSD support both "source directory" and "preserve files"
+  * options, OpenBSD does not...
+  */
+
+#ifdef __FreeBSD__
+  fprintf(fp, "@srcdir %s/%s/%s.buildroot\n", current, directory, prodfull);
   fputs("@option preserve\n", fp);
+#elif defined(__NetBSD__)
+  fprintf(fp, "@src %s/%s/%s.buildroot\n", current, directory, prodfull);
+  fputs("@option preserve\n", fp);
+#endif /* __FreeBSD__ */
 
   for (i = dist->num_depends, d = dist->depends; i > 0; i --, d ++)
   {
+    if (d->subpackage != subpackage)
+      continue;
+
     if (d->type == DEPEND_REQUIRES)
+#ifdef __OpenBSD__
+      fprintf(fp, "@depend %s", d->product);
+#else
       fprintf(fp, "@pkgdep %s", d->product);
+#endif /* __OpenBSD__ */
     else
 #ifdef __FreeBSD__
      /*
-      * FreeBSD doesn't have @pkgcfl command...
+      * FreeBSD uses @conflicts...
       */
-      fprintf(fp, "@comment conflicts with: %s", d->product);
+      fprintf(fp, "@conflicts %s", d->product);
+#elif defined(__OpenBSD__)
+      fprintf(fp, "@conflict %s", d->product);
 #else
       fprintf(fp, "@pkgcfl %s", d->product);
-#endif /* __FreeBSD */
+#endif /* __FreeBSD__ */
     if (d->vernumber[0] == 0)
     {
       if (d->vernumber[1] < INT_MAX)
@@ -172,22 +238,37 @@ make_bsd(const char     *prodname,	/* I - Product short name */
   }
 
   for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
-    switch (c->type)
+    if (c->subpackage == subpackage)
+      switch (c->type)
+      {
+	case COMMAND_PRE_INSTALL :
+            fputs("WARNING: Package contains pre-install commands which are not supported\n"
+	          "         by the BSD packager.\n", stderr);
+            break;
+	case COMMAND_POST_INSTALL :
+            fprintf(fp, "@exec %s\n", c->command);
+	    break;
+	case COMMAND_PRE_REMOVE :
+            fprintf(fp, "@unexec %s\n", c->command);
+	    break;
+	case COMMAND_POST_REMOVE :
+            fputs("WARNING: Package contains post-removal commands which are not supported\n"
+	          "         by the BSD packager.\n", stderr);
+            break;
+      }
+
+  for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
+    if (tolower(file->type) == 'd' && file->subpackage == subpackage)
     {
-      case COMMAND_PRE_INSTALL :
-          fputs("WARNING: Package contains pre-install commands which are not supported\n"
-	        "         by the BSD packager.\n", stderr);
-          break;
-      case COMMAND_POST_INSTALL :
-          fprintf(fp, "@exec %s\n", c->command);
-	  break;
-      case COMMAND_PRE_REMOVE :
-          fprintf(fp, "@unexec %s\n", c->command);
-	  break;
-      case COMMAND_POST_REMOVE :
-          fputs("WARNING: Package contains post-removal commands which are not supported\n"
-	        "         by the BSD packager.\n", stderr);
-          break;
+     /*
+      * We create and update directories as postinstall commands to
+      * avoid a bug in the FreeBSD pkg_delete command.
+      */
+
+      fprintf(fp, "@exec /bin/mkdir -p %s\n", file->dst);
+      fprintf(fp, "@exec /bin/chown %s:%s %s\n", file->user, file->group,
+              file->dst);
+      fprintf(fp, "@exec /bin/chmod %04o %s\n", file->mode, file->dst);
     }
 
   for (i = dist->num_files, file = dist->files, old_mode = 0, old_user = "",
@@ -195,6 +276,16 @@ make_bsd(const char     *prodname,	/* I - Product short name */
        i > 0;
        i --, file ++)
   {
+   /*
+    * The FreeBSD pkg_delete command (at least) doesn't like creating
+    * and deleting directories.  I don't know if other BSD's have the
+    * same problem, but for now just put the directory stuff in a
+    * postinstall script...
+    */
+
+    if (tolower(file->type) == 'd' || file->subpackage != subpackage)
+      continue;
+
     if (file->mode != old_mode)
       fprintf(fp, "@mode %04o\n", old_mode = file->mode);
     if (strcmp(file->user, old_user))
@@ -210,10 +301,13 @@ make_bsd(const char     *prodname,	/* I - Product short name */
       case 'c' :
       case 'f' :
       case 'l' :
+#ifdef __OpenBSD__
+          qprintf(fp, "@file %s\n", file->dst + 1);
+	  if (tolower(file->type) == 'c')
+            qprintf(fp, "@sample %s\n", file->dst + 1);
+#else
           qprintf(fp, "%s\n", file->dst + 1);
-          break;
-      case 'd' :
-          qprintf(fp, "%s\n", file->dst + 1);
+#endif /* __OpenBSD__ */
           break;
     }
   }
@@ -226,7 +320,7 @@ make_bsd(const char     *prodname,	/* I - Product short name */
   for (i = dist->num_files, file = dist->files + i - 1;
        i > 0;
        i --, file --)
-    if (tolower(file->type) == 'd')
+    if (tolower(file->type) == 'd' && file->subpackage == subpackage)
       qprintf(fp, "@dirrm %s\n", file->dst + 1);
 
   fclose(fp);
@@ -240,6 +334,9 @@ make_bsd(const char     *prodname,	/* I - Product short name */
 
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
   {
+    if (file->subpackage != subpackage)
+      continue;
+
    /*
     * Find the username and groupname IDs...
     */
@@ -258,7 +355,8 @@ make_bsd(const char     *prodname,	/* I - Product short name */
     {
       case 'c' :
       case 'f' :
-          snprintf(filename, sizeof(filename), "%s/buildroot%s", directory, file->dst);
+          snprintf(filename, sizeof(filename), "%s/%s.buildroot%s",
+	           directory, prodfull, file->dst);
 
 	  if (Verbosity > 1)
 	    printf("%s -> %s...\n", file->src, filename);
@@ -268,8 +366,8 @@ make_bsd(const char     *prodname,	/* I - Product short name */
 	    return (1);
           break;
       case 'i' :
-          snprintf(filename, sizeof(filename), "%s/buildroot/etc/rc.d/%s", directory,
-	          file->dst);
+          snprintf(filename, sizeof(filename), "%s/%s.buildroot/etc/rc.d/%s",
+	           directory, prodfull, file->dst);
 
 	  if (Verbosity > 1)
 	    printf("%s -> %s...\n", file->src, filename);
@@ -279,7 +377,8 @@ make_bsd(const char     *prodname,	/* I - Product short name */
 	    return (1);
           break;
       case 'd' :
-          snprintf(filename, sizeof(filename), "%s/buildroot%s", directory, file->dst);
+          snprintf(filename, sizeof(filename), "%s/%s.buildroot%s",
+	           directory, prodfull, file->dst);
 
 	  if (Verbosity > 1)
 	    printf("Directory %s...\n", filename);
@@ -288,7 +387,8 @@ make_bsd(const char     *prodname,	/* I - Product short name */
 			 grp ? grp->gr_gid : 0);
           break;
       case 'l' :
-          snprintf(filename, sizeof(filename), "%s/buildroot%s", directory, file->dst);
+          snprintf(filename, sizeof(filename), "%s/%s.buildroot%s",
+	           directory, prodfull, file->dst);
 
 	  if (Verbosity > 1)
 	    printf("%s -> %s...\n", file->src, filename);
@@ -303,14 +403,35 @@ make_bsd(const char     *prodname,	/* I - Product short name */
   */
 
   if (Verbosity)
-    puts("Building FreeBSD pkg binary distribution...");
+    printf("Building %s *BSD pkg binary distribution...\n", prodfull);
 
-  if (run_command(NULL, "pkg_create -p / -s %s -c %s -d %s -f %s %s",
-                  current, commentname, descrname, plistname, name))
+#ifdef __OpenBSD__
+  if (run_command(NULL, "pkg_create -p / -B %s/%s.buildroot "
+                        "-c %s "
+			"-d %s "
+                        "-f %s "
+			"%s.tgz",
+                  directory, prodfull,
+		  commentname,
+		  descrname,
+		  plistname,
+		  name))
     return (1);
 
   if (run_command(NULL, "mv %s.tgz %s", name, directory))
     return (1);
+#else
+  if (run_command(NULL, "pkg_create -p / "
+                        "-c %s "
+			"-d %s "
+                        "-f %s "
+			"%s/%s.tgz",
+		  commentname,
+		  descrname,
+		  plistname,
+		  directory, name))
+    return (1);
+#endif /* __OpenBSD__ */
 
  /*
   * Remove temporary files...
@@ -321,7 +442,7 @@ make_bsd(const char     *prodname,	/* I - Product short name */
     if (Verbosity)
       puts("Removing temporary distribution files...");
 
-    run_command(NULL, "/bin/rm -rf %s/buildroot", directory);
+    run_command(NULL, "/bin/rm -rf %s/%s.buildroot", directory, prodfull);
 
     unlink(plistname);
     unlink(commentname);
@@ -333,5 +454,5 @@ make_bsd(const char     *prodname,	/* I - Product short name */
 
 
 /*
- * End of "$Id: bsd.c,v 1.9 2002/10/17 18:13:13 mike Exp $".
+ * End of "$Id: bsd.c 670 2006-04-05 15:08:08Z mike $".
  */
