@@ -1,5 +1,5 @@
 /*
- * "$Id: deb.c 670 2006-04-05 15:08:08Z mike $"
+ * "$Id: deb.c 697 2006-08-17 21:00:25Z mike $"
  *
  *   Debian package gateway for the ESP Package Manager (EPM).
  *
@@ -117,6 +117,26 @@ make_deb(const char     *prodname,	/* I - Product short name */
     tar_close(tarfile);
   }
 
+ /*
+  * Remove temporary files...
+  */
+
+  if (!KeepFiles && dist->num_subpackages)
+  {
+    if (Verbosity)
+      puts("Removing temporary distribution files...");
+
+   /*
+    * Remove .deb files since they are now in a .tgz file...
+    */
+
+    unlink_package("deb", prodname, directory, platname, dist, NULL);
+
+    for (i = 0; i < dist->num_subpackages; i ++)
+      unlink_package("deb", prodname, directory, platname, dist,
+                     dist->subpackages[i]);
+  }
+
   return (0);
 }
 
@@ -577,5 +597,5 @@ make_subpackage(const char     *prodname,
 
 
 /*
- * End of "$Id: deb.c 670 2006-04-05 15:08:08Z mike $".
+ * End of "$Id: deb.c 697 2006-08-17 21:00:25Z mike $".
  */
