@@ -1,5 +1,5 @@
 /*
- * "$Id: dist.c 670 2006-04-05 15:08:08Z mike $"
+ * "$Id: dist.c 733 2006-11-30 21:59:27Z mike $"
  *
  *   Distribution functions for the ESP Package Manager (EPM).
  *
@@ -39,7 +39,6 @@
  *   get_inline()       - Read inline lines into a string...
  *   get_line()         - Get a line from a file, filtering for uname lines...
  *   get_string()       - Get a delimited string from a line.
- *   get_vernumber()    - Convert a version string to a number...
  *   patmatch()         - Pattern matching...
  *   sort_subpackages() - Compare two subpackage names.
  */
@@ -74,7 +73,6 @@ static char	*get_line(char *buffer, int size, FILE *fp,
 		          struct utsname *platform, const char *format,
 			  int *skip);
 static char	*get_string(char **src, char *dst, size_t dstsize);
-static int	get_vernumber(const char *version);
 static int	patmatch(const char *, const char *);
 static int	sort_subpackages(char **a, char **b);
 
@@ -394,12 +392,6 @@ add_subpackage(dist_t     *dist,	/* I - Distribution */
     qsort(dist->subpackages, (size_t)dist->num_subpackages, sizeof(char *),
           (int (*)(const void *, const void *))sort_subpackages);
 
- /*
-  * Add a dependency on the base package...
-  */
-
-  add_depend(dist, DEPEND_REQUIRES, "_self", s);
-
  /*
   * Return the new string...
   */
@@ -529,8 +521,16 @@ get_platform(struct utsname *platform)	/* O - Platform info */
   char	line[1024],			/* Line from file */
 	*ptr;				/* Pointer into line */
   int	major, minor;			/* Major and minor release numbers */
+#endif /* __APPLE__ */
 
 
+ /*
+  * Get the system identification information...
+  */
+
+  uname(platform);
+
+#ifdef __APPLE__
  /*
   * Try to get the OS version number from the SystemVersion.plist file.
   * If not present, use the uname() results for Darwin...
@@ -569,13 +569,7 @@ get_platform(struct utsname *platform)	/* O - Platform info */
 
     strcpy(platform->sysname, "macosx");
   }
-  else
 #endif /* __APPLE__ */
- /*
-  * Get the system identification information...
-  */
-
-  uname(platform);
 
  /*
   * Adjust the CPU type accordingly...
@@ -934,10 +928,22 @@ read_dist(const char     *filename,	/* I - Main distribution list file */
 	{
 	  subpkg = find_subpackage(dist, temp);
 	}
-	else if (strcmp(line, "%version") == 0)
+	else if (!strcmp(line, "%version"))
 	{
           if (!dist->version[0])
 	  {
+	    if (strchr(temp, ':'))
+	    {
+	     /*
+	      * Grab the epoch...
+	      */
+
+	      dist->epoch = strtol(temp, &temp, 10);
+
+	      if (*temp == ':')
+	        temp ++;
+	    }
+
             strlcpy(dist->version, temp, sizeof(dist->version));
 	    if ((temp = strchr(dist->version, ' ')) != NULL)
 	    {
@@ -2130,84 +2136,6 @@ get_string(char   **src,		/* IO - Source string */
 }
 
 
-/*
- * 'get_vernumber()' - Convert a version string to a number...
- */
-
-static int				/* O - Version number */
-get_vernumber(const char *version)	/* I - Version string */
-{
-  int		numbers[4],		/* Raw version numbers */
-		nnumbers,		/* Number of numbers in version */
-		temp,			/* Temporary version number */
-		offset;			/* Offset for last version number */
-  const char	*ptr;			/* Pointer into string */
-
-
- /*
-  * Loop through the version number string and construct a version number.
-  */
-
-  memset(numbers, 0, sizeof(numbers));
-
-  for (ptr = version; *ptr && !isdigit(*ptr & 255); ptr ++);
-    /* Skip leading letters, periods, etc. */
-
-  for (offset = 0, temp = 0, nnumbers = 0; *ptr && !isspace(*ptr & 255); ptr ++)
-    if (isdigit(*ptr & 255))
-      temp = temp * 10 + *ptr - '0';
-    else
-    {
-     /*
-      * Add each mini version number (m.n.p) and patch/pre stuff...
-      */
-
-      if (nnumbers < 4)
-      {
-        numbers[nnumbers] = temp;
-	nnumbers ++;
-      }
-
-      temp = 0;
-
-      if (*ptr == '.')
-	offset = 0;
-      else if (*ptr == 'p' || *ptr == '-')
-      {
-	if (strncmp(ptr, "pre", 3) == 0)
-	{
-	  ptr += 2;
-	  offset = -20;
-	}
-	else
-	  offset = 0;
-
-        nnumbers = 3;
-      }
-      else if (*ptr == 'b')
-      {
-	offset   = -50;
-        nnumbers = 3;
-      }
-      else /* if (*ptr == 'a') */
-      {
-	offset   = -100;
-        nnumbers = 3;
-      }
-    }
-
-  if (nnumbers < 4)
-    numbers[nnumbers] = temp;
-
- /*
-  * Compute the version number as MMmmPPpp + offset
-  */
-
-  return (((numbers[0] * 100 + numbers[1]) * 100 + numbers[2]) * 100 +
-          numbers[3] + offset);
-}
-
-
 /*
  * 'patmatch()' - Pattern matching...
  */
@@ -2337,5 +2265,5 @@ sort_subpackages(char **a,		/* I - First subpackage */
 
 
 /*
- * End of "$Id: dist.c 670 2006-04-05 15:08:08Z mike $".
+ * End of "$Id: dist.c 733 2006-11-30 21:59:27Z mike $".
  */
