@@ -1,9 +1,9 @@
 /*
- * "$Id: epminstall.c,v 1.10 2002/08/30 02:00:42 mike Exp $"
+ * "$Id: epminstall.c 611 2005-01-11 21:37:42Z mike $"
  *
  *   Install program replacement for the ESP Package Manager (EPM).
  *
- *   Copyright 1999-2002 by Easy Software Products.
+ *   Copyright 1999-2005 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
  *   find_file()  - Find a file in the distribution...
  *   info()       - Show the EPM copyright and license.
  *   usage()      - Show command-line usage instructions.
- *   write_dist() - Write a distribution list file...
  */
 
 /*
@@ -45,7 +44,6 @@ int		Verbosity = 0;
 static file_t	*find_file(dist_t *dist, const char *dst);
 static void	info(void);
 static void	usage(void);
-static int	write_dist(const char *listname, dist_t *dist);
 
 
 /*
@@ -170,13 +168,13 @@ main(int  argc,			/* I - Number of command-line arguments */
       */
 
       if ((file = find_file(dist, files[i])) == NULL)
-        file = add_file(dist);
+        file = add_file(dist, NULL);
 
       file->type = 'd';
       file->mode = mode & 07777;
-      strncpy(file->user, user, sizeof(file->user) - 1);
-      strncpy(file->group, group, sizeof(file->group) - 1);
-      strncpy(file->dst, files[i], sizeof(file->dst) - 1);
+      strlcpy(file->user, user, sizeof(file->user));
+      strlcpy(file->group, group, sizeof(file->group));
+      strlcpy(file->dst, files[i], sizeof(file->dst));
       strcpy(file->src, "-");
     }
   }
@@ -194,7 +192,7 @@ main(int  argc,			/* I - Number of command-line arguments */
       if (file == NULL || file->type != 'd')
       {
         if (!file)
-	  file = add_file(dist);
+	  file = add_file(dist, NULL);
 
         if (stat(files[0], &fileinfo))
 	{
@@ -229,10 +227,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	else
 	  file->mode = 0644;
 
-	strncpy(file->user, user, sizeof(file->user) - 1);
-	strncpy(file->group, group, sizeof(file->group) - 1);
-	strncpy(file->dst, files[1], sizeof(file->dst) - 1);
-	strncpy(file->src, files[0], sizeof(file->src) - 1);
+	strlcpy(file->user, user, sizeof(file->user));
+	strlcpy(file->group, group, sizeof(file->group));
+	strlcpy(file->dst, files[1], sizeof(file->dst));
+	strlcpy(file->src, files[0], sizeof(file->src));
       }
       else
         num_files --;
@@ -249,13 +247,13 @@ main(int  argc,			/* I - Number of command-line arguments */
         * Add the installation directory to the file list...
 	*/
 
-	file = add_file(dist);
+	file = add_file(dist, NULL);
 
 	file->type = 'd';
 	file->mode = 0755;
-	strncpy(file->user, user, sizeof(file->user) - 1);
-	strncpy(file->group, group, sizeof(file->group) - 1);
-	strncpy(file->dst, files[num_files], sizeof(file->dst) - 1);
+	strlcpy(file->user, user, sizeof(file->user));
+	strlcpy(file->group, group, sizeof(file->group));
+	strlcpy(file->dst, files[num_files], sizeof(file->dst));
 	strcpy(file->src, "-");
       }
       else if (file->type != 'd')
@@ -279,7 +277,7 @@ main(int  argc,			/* I - Number of command-line arguments */
         snprintf(dst, sizeof(dst), "%s/%s", files[num_files], src);
 
 	if ((file = find_file(dist, dst)) == NULL)
-          file = add_file(dist);
+          file = add_file(dist, NULL);
 
         if (stat(files[i], &fileinfo))
 	{
@@ -314,10 +312,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	else
 	  file->mode = 0644;
 
-	strncpy(file->user, user, sizeof(file->user) - 1);
-	strncpy(file->group, group, sizeof(file->group) - 1);
-	strncpy(file->dst, dst, sizeof(file->dst) - 1);
-	strncpy(file->src, files[i], sizeof(file->src) - 1);
+	strlcpy(file->user, user, sizeof(file->user));
+	strlcpy(file->group, group, sizeof(file->group));
+	strlcpy(file->dst, dst, sizeof(file->dst));
+	strlcpy(file->src, files[i], sizeof(file->src));
       }
     }
   }
@@ -375,7 +373,7 @@ static void
 info(void)
 {
   puts(EPM_VERSION);
-  puts("Copyright 1999-2002 by Easy Software Products.");
+  puts("Copyright 1999-2005 by Easy Software Products.");
   puts("");
   puts("EPM is free software and comes with ABSOLUTELY NO WARRANTY; for details");
   puts("see the GNU General Public License in the file COPYING or at");
@@ -409,143 +407,5 @@ usage(void)
 
 
 /*
- * 'write_dist()' - Write a distribution list file...
- */
-
-static int				/* O - 0 on success, -1 on failure */
-write_dist(const char *listname,	/* I - File to write to */
-           dist_t     *dist)		/* I - Distribution to write */
-{
-  int		i;			/* Looping var */
-  int		is_inline;		/* Inline text? */
-  char		listbck[1024],		/* Backup filename */
-		*ptr;			/* Pointer into command string */
-  FILE		*listfile;		/* Output file */
-  file_t	*file;			/* Current file entry */
-  static const char *commands[] =	/* Command strings */
-		{
-		  "%preinstall",
-		  "%postinstall",
-		  "%prepatch",
-		  "%postpatch",
-		  "%preremove",
-		  "%postremove"
-		},
-		*depends[] =		/* Dependency strings */
-		{
-		  "%requires",
-		  "%incompat",
-		  "%replaces",
-		  "%provides"
-		};
-
-
- /*
-  * Make a backup of the list file...
-  */
-
-  snprintf(listbck, sizeof(listbck), "%s.O", listname);
-
-  rename(listname, listbck);
-
- /*
-  * Open the list file...
-  */
-
-  if ((listfile = fopen(listname, "w")) == NULL)
-  {
-    rename(listbck, listname);
-    return (-1);
-  }
-
- /*
-  * Write the list file...
-  */
-
-  fputs("# List file created by epminstall\n", listfile);
-  fputs("# " EPM_VERSION "\n", listfile);
-
-  if (dist->product[0])
-    fprintf(listfile, "%%product %s\n", dist->product);
-  if (dist->version[0])
-    fprintf(listfile, "%%version %s %d\n", dist->version, dist->vernumber);
-  if (dist->relnumber)
-    fprintf(listfile, "%%release %d\n", dist->relnumber);
-  if (dist->copyright[0])
-    fprintf(listfile, "%%copyright %s\n", dist->copyright);
-  if (dist->vendor[0])
-    fprintf(listfile, "%%vendor %s\n", dist->vendor);
-  if (dist->packager[0])
-    fprintf(listfile, "%%packager %s\n", dist->packager);
-  if (dist->license[0])
-    fprintf(listfile, "%%license %s\n", dist->license);
-  if (dist->readme[0])
-    fprintf(listfile, "%%readme %s\n", dist->readme);
-
-  for (i = 0; i < dist->num_descriptions; i ++)
-    if (strchr(dist->descriptions[i], '\n') != NULL)
-      fprintf(listfile, "%%description <<EPM-END-INLINE\n%s\nEPM-END-INLINE\n", dist->descriptions[i]);
-    else
-      fprintf(listfile, "%%description %s\n", dist->descriptions[i]);
-
-  for (i = 0; i < dist->num_depends; i ++)
-  {
-    fprintf(listfile, "%s %s", depends[(int)dist->depends[i].type],
-            dist->depends[i].product);
-
-    if (dist->depends[i].version[0][0] ||
-        dist->depends[i].version[1][0])
-    {
-      fprintf(listfile, " %s %d %s %d\n",
-              dist->depends[i].version[0],
-	      dist->depends[i].vernumber[0],
-              dist->depends[i].version[1],
-	      dist->depends[i].vernumber[1]);
-    }
-    else
-      putc('\n', listfile);
-  }
-
-  for (i = 0; i < dist->num_commands; i ++)
-  {
-    fputs(commands[(int)dist->commands[i].type], listfile);
-
-    is_inline = strchr(dist->commands[i].command, '\n') != NULL;
-
-    if (is_inline)
-      fputs(" <<EPM-END-INLINE\n", listfile);
-    else
-      putc(' ', listfile);
-
-    for (ptr = dist->commands[i].command; *ptr; ptr ++)
-    {
-      if (*ptr == '$' && !is_inline)
-        putc(*ptr, listfile);
-
-      putc(*ptr, listfile);
-    }
-    putc('\n', listfile);
-
-    if (is_inline)
-      fputs("EPM-END-INLINE\n", listfile);
-  }
-
-  for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
-  {
-    qprintf(listfile, "%c %04o %s %s %s %s",
-	    file->type, file->mode, file->user, file->group,
-	    file->dst, file->src);
-
-    if (file->options[0])
-      qprintf(listfile, "%s\n", file->options);
-    else
-      putc('\n', listfile);
-  }
-
-  return (fclose(listfile));
-}
-
-
-/*
- * End of "$Id: epminstall.c,v 1.10 2002/08/30 02:00:42 mike Exp $".
+ * End of "$Id: epminstall.c 611 2005-01-11 21:37:42Z mike $".
  */
