@@ -1,9 +1,9 @@
 /*
- * "$Id: osx.c 670 2006-04-05 15:08:08Z mike $"
+ * "$Id: osx.c 699 2006-08-21 21:04:23Z mike $"
  *
  *   MacOS X package gateway for the ESP Package Manager (EPM).
  *
- *   Copyright 2002-2005 by Easy Software Products.
+ *   Copyright 2002-2006 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -221,6 +221,8 @@ make_osx(const char     *prodname,	/* I - Product short name */
       fputs("                </dict>\n", fp);
     }
     fputs("        </array>\n", fp);
+    fputs("        <key>CFBundleIdentifier</key>\n", fp);
+    fprintf(fp, "        <string>%s</string>\n", prodname);
     fputs("        <key>CFBundleName</key>\n", fp);
     fprintf(fp, "        <string>%s</string>\n", dist->product);
     fputs("        <key>CFBundleGetInfoString</key>\n", fp);
@@ -251,12 +253,14 @@ make_osx(const char     *prodname,	/* I - Product short name */
     if (pm_paths[i])
       run_command(NULL, "%s/Contents/MacOS/PackageMaker -build "
 			"-p %s/%s.mpkg "
+//		        "-mi %s/Packages "
 		        "-f %s/MetaPackage "
 			"-r %s/MetaResources "
 			"-d %s/%s-metadesc.plist "
 			"-i %s/%s-metainfo.plist",
 		  pm_paths[i],
 		  filename, prodname,
+//		  filename,
 		  filename,
 		  filename,
 		  filename, prodname,
@@ -859,5 +863,5 @@ write_rtf(dist_t     *dist,		/* I - Distribution  information */
 }
 
 /*
- * End of "$Id: osx.c 670 2006-04-05 15:08:08Z mike $".
+ * End of "$Id: osx.c 699 2006-08-21 21:04:23Z mike $".
  */
