@@ -1,9 +1,9 @@
 /*
- * "$Id: setld.c,v 1.14 2002/10/17 18:13:14 mike Exp $"
+ * "$Id: setld.c 670 2006-04-05 15:08:08Z mike $"
  *
  *   Tru64 package gateway for the ESP Package Manager (EPM)
  *
- *   Copyright 2001-2002 by Easy Software Products
+ *   Copyright 2001-2006 by Easy Software Products
  *   Copyright 2001 by Aneesh Kumar (aneesh.kumar@digital.com) at Digital India.
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -39,7 +39,7 @@ make_setld(const char     *prodname,	/* I - Product short name */
            dist_t         *dist,	/* I - Distribution information */
            struct utsname *platform)	/* I - Platform information */
 {
-  int		i;			/* Looping var */
+  int		i, j;			/* Looping vars */
   FILE		*fp;			/* Spec file */
   tarf_t	*tarfile;		/* .tardist file */
   char		name[1024];		/* Full product name */
@@ -56,7 +56,7 @@ make_setld(const char     *prodname,	/* I - Product short name */
   const char	*runlevels;		/* Run levels */
 
 
-  (void)platform;
+  REF(platform);
 
  /*
   * Check the package information to make sure it complies with the
@@ -73,33 +73,53 @@ make_setld(const char     *prodname,	/* I - Product short name */
 
   if (strlen(prodname) < 3)
   {
-    fprintf(stderr, "epm: Need a product name of at least 3 characters.\n"
+    fprintf(stderr, "epm: Need a product name of at least 3 uppercase characters.\n"
                     "     The current product name (%s) is not acceptable.\n",
             prodname);
     return (1);
   }
 
   for (i = 0; prodname[i]; i ++)
-    if (!isupper(prodname[i]))
+    if (!isupper(prodname[i] & 255))
     {
-      fprintf(stderr, "epm: Need a product name containing uppercase letters.\n"
+      fprintf(stderr, "epm: Need a product name of at least 3 uppercase characters.\n"
                       "     The current product name (%s) is not acceptable.\n",
               prodname);
       return (1);
     }
 
+  for (i = 0; i < dist->num_subpackages; i ++)
+  {
+    if ((strlen(dist->subpackages[i]) + strlen(prodname)) > 77)
+    {
+      fprintf(stderr, "epm: Product + subpackage names must be less than 77 characters.\n"
+                      "     The current subpackage name (%s) is not acceptable.\n",
+              dist->subpackages[i]);
+      return (1);
+    }
+
+    for (j = 0; dist->subpackages[i][j]; j ++)
+      if (!isupper(dist->subpackages[i][j] & 255))
+      {
+	fprintf(stderr, "epm: Subpackage names may only contain uppercase letters.\n"
+                	"     The current subpackage name (%s) is not acceptable.\n",
+        	dist->subpackages[i]);
+	return (1);
+      }
+  }
+
  /*
   * Prepare for packaging...
   */
 
-  if (dist->relnumber)
+  if (dist->release[0])
   {
     if (platname[0])
-      snprintf(name, sizeof(name), "%s-%s-%d-%s", prodname, dist->version,
-               dist->relnumber, platname);
+      snprintf(name, sizeof(name), "%s-%s-%s-%s", prodname, dist->version,
+               dist->release, platname);
     else
-      snprintf(name, sizeof(name), "%s-%s-%d", prodname, dist->version,
-               dist->relnumber);
+      snprintf(name, sizeof(name), "%s-%s-%s", prodname, dist->version,
+               dist->release);
   }
   else if (platname[0])
     snprintf(name, sizeof(name), "%s-%s-%s", prodname, dist->version, platname);
@@ -108,8 +128,6 @@ make_setld(const char     *prodname,	/* I - Product short name */
 
   getcwd(current, sizeof(current));
 
-  snprintf(subset, sizeof(subset), "%sALL%03d", prodname, dist->vernumber);
-
  /*
   * Add symlinks for init scripts...
   */
@@ -122,10 +140,10 @@ make_setld(const char     *prodname,	/* I - Product short name */
       */
 
       for (runlevels = get_runlevels(dist->files + i, "023");
-           isdigit(*runlevels);
+           isdigit(*runlevels & 255);
 	   runlevels ++)
       {
-	file = add_file(dist);
+	file = add_file(dist, dist->files[i].subpackage);
 	file->type = 'l';
 	file->mode = 0;
 	strcpy(file->user, "root");
@@ -186,7 +204,8 @@ make_setld(const char     *prodname,	/* I - Product short name */
   if (Verbosity)
     puts("Creating subset control program...");
 
-  snprintf(scpname, sizeof(scpname), "%s/src/scps/%s.scp", directory, subset);
+  snprintf(scpname, sizeof(scpname), "%s/src/scps/%sALL%03d.scp", directory,
+           prodname, dist->vernumber);
 
   if ((fp = fopen(scpname, "w")) == NULL)
   {
@@ -201,7 +220,11 @@ make_setld(const char     *prodname,	/* I - Product short name */
   fputs("PRE_L)\n", fp);
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
     if (tolower(file->type) == 'i')
-      qprintf(fp, "/sbin/init.d/%s stop\n", file->dst);
+    {
+      qprintf(fp, "if test -x %s; then\n", file->dst);
+      qprintf(fp, "	%s stop\n", file->dst);
+      fputs("fi\n", fp);
+    }
   for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
     if (c->type == COMMAND_PRE_INSTALL)
       fprintf(fp, "%s\n", c->command);
@@ -220,13 +243,13 @@ make_setld(const char     *prodname,	/* I - Product short name */
     }
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
     if (tolower(file->type) == 'i')
-      qprintf(fp, "/sbin/init.d/%s start\n", file->dst);
+      qprintf(fp, "%s start\n", file->dst);
   fputs(";;\n", fp);
 
   fputs("PRE_D)\n", fp);
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
     if (tolower(file->type) == 'i')
-      qprintf(fp, "/sbin/init.d/%s stop\n", file->dst);
+      qprintf(fp, "%s stop\n", file->dst);
   for (i = dist->num_commands, c = dist->commands; i > 0; i --, c ++)
     if (c->type == COMMAND_PRE_REMOVE)
       fprintf(fp, "%s\n", c->command);
@@ -250,7 +273,7 @@ make_setld(const char     *prodname,	/* I - Product short name */
   fclose(fp);
 
  /*
-  * Sort the file list by the destination name, since kits needs a sorted
+  * Sort the file list by the destination name, since kits need a sorted
   * list...
   */
 
@@ -273,6 +296,10 @@ make_setld(const char     *prodname,	/* I - Product short name */
   }
 
   for (i = dist->num_files, file = dist->files; i > 0; i --, file ++)
+  {
+    snprintf(subset, sizeof(subset), "%s%s%03d", prodname,
+             file->subpackage ? file->subpackage : "ALL", dist->vernumber);
+
     switch (tolower(file->type))
     {
       case 'c' :
@@ -285,6 +312,7 @@ make_setld(const char     *prodname,	/* I - Product short name */
           fprintf(fp, "0\t.%s\t%s\n", file->dst, subset);
 	  break;
     }
+  }
 
   fclose(fp);
 
@@ -295,7 +323,8 @@ make_setld(const char     *prodname,	/* I - Product short name */
   if (Verbosity)
     puts("Creating key file...");
 
-  snprintf(keyname, sizeof(keyname), "%s/src/%s%03d.k", directory, prodname, dist->vernumber);
+  snprintf(keyname, sizeof(keyname), "%s/src/%s%03d.k", directory, prodname,
+           dist->vernumber);
 
   if ((fp = fopen(keyname, "w")) == NULL)
   {
@@ -310,7 +339,11 @@ make_setld(const char     *prodname,	/* I - Product short name */
   fprintf(fp, "MI=%s%03d.mi\n", prodname, dist->vernumber);
   fputs("COMPRESS=0\n", fp);
   fputs("%%\n", fp);
-  qprintf(fp, "%s\t.\t0\t'%s, %s'\n", subset, dist->product, dist->version);
+  qprintf(fp, "%sALL%03d\t.\t0\t'%s, %s'\n", prodname, dist->vernumber,
+          dist->product, dist->version);
+  for (i = 0; i < dist->num_subpackages; i ++)
+    qprintf(fp, "%s%s%03d\t.\t0\t'%s, %s'\n", prodname, dist->subpackages[i],
+            dist->vernumber, dist->product, dist->version);
   fclose(fp);
 
  /*
@@ -428,5 +461,5 @@ make_setld(const char     *prodname,	/* I - Product short name */
 
 
 /*
- * End of "$Id: setld.c,v 1.14 2002/10/17 18:13:14 mike Exp $".
+ * End of "$Id: setld.c 670 2006-04-05 15:08:08Z mike $".
  */
