@@ -1,9 +1,9 @@
 //
-// "$Id: setup2.cxx 645 2005-10-14 18:01:51Z mike $"
+// "$Id: setup2.cxx 739 2006-12-05 17:20:27Z mike $"
 //
 //   ESP Software Installation Wizard main entry for the ESP Package Manager (EPM).
 //
-//   Copyright 1999-2005 by Easy Software Products.
+//   Copyright 1999-2006 by Easy Software Products.
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License as published by
@@ -20,8 +20,10 @@
 //   main()         - Main entry for software wizard...
 //   get_dists()    - Get a list of available software products.
 //   install_dist() - Install a distribution...
+//   license_dist() - Show the license for a distribution...
 //   list_cb()      - Handle selections in the software list.
 //   load_image()   - Load the setup image file (setup.gif/xpm)...
+//   load_readme()  - Load the readme file...
 //   load_types()   - Load the installation types from the setup.types file.
 //   log_cb()       - Add one or more lines of text to the installation log.
 //   next_cb()      - Show software selections or install software.
@@ -75,6 +77,18 @@ AuthorizationRef SetupAuthorizationRef;
 #endif // __APPLE__
 
 
+//
+// Panes...
+//
+
+#define PANE_WELCOME	0
+#define PANE_TYPE	1
+#define PANE_SELECT	2
+#define PANE_CONFIRM	3
+#define PANE_LICENSE	4
+#define PANE_INSTALL	5
+
+
 //
 // Define a C API function type for comparisons...
 //
@@ -89,8 +103,10 @@ typedef int (*compare_func_t)(const void *, const void *);
 //
 
 void	get_dists(const char *d);
-int	install_dist(const dist_t *dist);
+int	install_dist(const gui_dist_t *dist);
+int	license_dist(const gui_dist_t *dist);
 void	load_image(void);
+void	load_readme(void);
 void	load_types(void);
 void	log_cb(int fd, int *fdptr);
 void	update_sizes(void);
@@ -108,10 +124,9 @@ main(int  argc,			// I - Number of command-line arguments
   const char	*distdir;	// Distribution directory
 
 
-#if !defined(__hpux) && !defined(__sun) && !defined(__osf) && !defined(_AIX)
-  // Use modern "skin" for modern OS's...
-  Fl::scheme("plastic");
-#endif // !__hpux && !__sun && !__osf && !_AIX
+  // Use GTK+ scheme for all operating systems...
+  Fl::background(230, 230, 230);
+  Fl::scheme("gtk+");
 
 #ifdef __APPLE__
   // OSX passes an extra command-line option when run from the Finder.
@@ -146,9 +161,6 @@ main(int  argc,			// I - Number of command-line arguments
   else
     distdir = ".";
 #else
-  // Use the default scheme on this system...
-  Fl::scheme(NULL);
-
   // Get the directory that has the software in it...
   if (argc > 1)
     distdir = argv[1];
@@ -158,13 +170,18 @@ main(int  argc,			// I - Number of command-line arguments
 
   w = make_window();
 
-  WelcomePane->show();
+  Pane[PANE_WELCOME]->show();
   PrevButton->deactivate();
   NextButton->deactivate();
 
-  load_image();
+  gui_get_installed();
+  get_dists(distdir);
 
-  w->show(1, argv);
+  load_image();
+  load_readme();
+  load_types();
+
+  w->show();
 
   while (!w->visible())
     Fl::wait();
@@ -203,11 +220,6 @@ main(int  argc,			// I - Number of command-line arguments
   }
 #endif // __APPLE__
 
-  get_installed();
-  get_dists(distdir);
-
-  load_types();
-
   NextButton->activate();
 
   Fl::run();
@@ -231,7 +243,7 @@ get_dists(const char *d)	// I - Directory to look in
   int		num_files;	// Number of files
   dirent	**files;	// Files
   const char	*ext;		// Extension
-  dist_t	*temp,		// Pointer to current distribution
+  gui_dist_t	*temp,		// Pointer to current distribution
 		*installed;	// Pointer to installed product
   FILE		*fp;		// File to read from
   char		line[1024],	// Line from file...
@@ -254,7 +266,7 @@ get_dists(const char *d)	// I - Directory to look in
 
   // Build a distribution list...
   NumDists = 0;
-  Dists    = (dist_t *)0;
+  Dists    = (gui_dist_t *)0;
 
   for (i = 0; i < num_files; i ++)
   {
@@ -269,7 +281,9 @@ get_dists(const char *d)	// I - Directory to look in
       }
 
       // Add a new distribution entry...
-      temp = add_dist(&NumDists, &Dists);
+      temp           = gui_add_dist(&NumDists, &Dists);
+      temp->type     = PACKAGE_PORTABLE;
+      temp->filename = strdup(files[i]->d_name);
 
       strncpy(temp->product, files[i]->d_name, sizeof(temp->product) - 1);
       *strrchr(temp->product, '.') = '\0';	// Drop .install
@@ -300,13 +314,68 @@ get_dists(const char *d)	// I - Directory to look in
 	  hiver  = 0;
 
 	  if (sscanf(line + 11, "%s%*s%d%*s%d", product, &lowver, &hiver) > 0)
-	    add_depend(temp, (line[2] == 'i') ? DEPEND_INCOMPAT : DEPEND_REQUIRES,
-	               product, lowver, hiver);
+	    gui_add_depend(temp, (line[2] == 'i') ? DEPEND_INCOMPAT :
+	                                            DEPEND_REQUIRES,
+	        	   product, lowver, hiver);
         }
       }
 
       fclose(fp);
     }
+    else if (!strcmp(ext, ".rpm"))
+    {
+      // Found an RPM package...
+      char	*version,		// Version number
+		*size,			// Size of package
+		*description;		// Summary string
+
+
+      // Get the package information...
+      snprintf(line, sizeof(line),
+               "rpm -qp --qf '%%{NAME}|%%{VERSION}|%%{SIZE}|%%{SUMMARY}\\n' %s",
+	       files[i]->d_name);
+
+      if ((fp = popen(line, "r")) == NULL)
+      {
+        free(files[i]);
+	continue;
+      }
+
+      if (!fgets(line, sizeof(line), fp))
+      {
+        free(files[i]);
+	continue;
+      }
+
+      // Drop the trailing newline...
+      line[strlen(line) - 1] = '\0';
+
+      // Grab the different fields...
+      if ((version = strchr(line, '|')) == NULL)
+        continue;
+      *version++ = '\0';
+
+      if ((size = strchr(version, '|')) == NULL)
+        continue;
+      *size++ = '\0';
+
+      if ((description = strchr(size, '|')) == NULL)
+        continue;
+      *description++ = '\0';
+
+      // Add a new distribution entry...
+      temp           = gui_add_dist(&NumDists, &Dists);
+      temp->type     = PACKAGE_RPM;
+      temp->filename = strdup(files[i]->d_name);
+
+      strlcpy(temp->product, line, sizeof(temp->product));
+      strlcpy(temp->name, description, sizeof(temp->name));
+      strlcpy(temp->version, version, sizeof(temp->version));
+      temp->vernumber = get_vernumber(version);
+      temp->rootsize  = (int)(atof(size) / 1024.0 + 0.5);
+
+      pclose(fp);
+    }
 
     free(files[i]);
   }
@@ -320,13 +389,14 @@ get_dists(const char *d)	// I - Directory to look in
   }
 
   if (NumDists > 1)
-    qsort(Dists, NumDists, sizeof(dist_t), (compare_func_t)sort_dists);
+    qsort(Dists, NumDists, sizeof(gui_dist_t), (compare_func_t)gui_sort_dists);
 
   for (i = 0, temp = Dists; i < NumDists; i ++, temp ++)
   {
     sprintf(line, "%s v%s", temp->name, temp->version);
 
-    if ((installed = find_dist(temp->product, NumInstalled, Installed)) == NULL)
+    if ((installed = gui_find_dist(temp->product, NumInstalled,
+                                   Installed)) == NULL)
     {
       strcat(line, " (new)");
       SoftwareList->add(line, 0);
@@ -357,7 +427,7 @@ get_dists(const char *d)	// I - Directory to look in
 //
 
 int					// O - Install status
-install_dist(const dist_t *dist)	// I - Distribution to install
+install_dist(const gui_dist_t *dist)	// I - Distribution to install
 {
   char		command[1024];		// Command string
   int		fds[2];			// Pipe FDs
@@ -365,58 +435,11 @@ install_dist(const dist_t *dist)	// I - Distribution to install
 #ifndef __APPLE__
   int		pid;			// Process ID
 #endif // !__APPLE__
-  char		licfile[1024];		// License filename
-  struct stat	licinfo;		// License file info
-  static int	liclength = 0;		// Size of license file
-  static char	liclabel[1024];		// Label for license pane
 
 
   sprintf(command, "**** %s ****", dist->name);
   InstallLog->add(command);
 
-  // See if we need to show the license file...
-  sprintf(licfile, "%s.license", dist->product);
-  if (!stat(licfile, &licinfo) && licinfo.st_size != liclength)
-  {
-    // Save this license's length...
-    liclength = licinfo.st_size;
-
-    // Set the title string...
-    snprintf(liclabel, sizeof(liclabel), "Software License for %s", dist->name);
-    LicensePane->label(liclabel);
-
-    // Load the license into the browser...
-    LicenseBrowser->clear();
-    LicenseBrowser->load(licfile);
-
-    // Show the license window and wait for the user...
-    LicensePane->show();
-    LicenseAccept->clear();
-    LicenseDecline->clear();
-    NextButton->deactivate();
-    CancelButton->activate();
-
-    while (LicensePane->visible())
-      Fl::wait();
-
-    CancelButton->deactivate();
-    NextButton->deactivate();
-
-    if (LicenseDecline->value())
-    {
-      // Can't install without acceptance...
-      liclength = 0;
-      snprintf(command, sizeof(command), "License not accepted for %s!",
-               dist->name);
-      InstallLog->add(command);
-      return (1);
-    }
-  }
-
-  sprintf(command, "%s.install", dist->product);
-  if (access(command, 0))
-    sprintf(command, "%s.patch", dist->product);
-
 #ifdef __APPLE__
   // Run the install script using Apple's authorization API...
   FILE		*fp = NULL;
@@ -424,7 +447,8 @@ install_dist(const dist_t *dist)	// I - Distribution to install
   OSStatus	astatus;
 
 
-  astatus = AuthorizationExecuteWithPrivileges(SetupAuthorizationRef, command,
+  astatus = AuthorizationExecuteWithPrivileges(SetupAuthorizationRef,
+                                               dist->filename,
                                                kAuthorizationFlagDefaults,
                                                args, &fp);
 
@@ -452,7 +476,11 @@ install_dist(const dist_t *dist)	// I - Distribution to install
     close(fds[1]);
 
     // Execute the command; if an error occurs, return it...
-    execl(command, command, "now", NULL);
+    if (dist->type == PACKAGE_PORTABLE)
+      execl(dist->filename, dist->filename, "now", (char *)0);
+    else
+      execlp("rpm", "rpm", "-U", "--nodeps", dist->filename, (char *)0);
+
     exit(errno);
   }
   else if (pid < 0)
@@ -517,6 +545,70 @@ install_dist(const dist_t *dist)	// I - Distribution to install
 }
 
 
+//
+// 'license_dist()' - Show the license for a distribution...
+//
+
+int					// O - 0 if accepted, 1 if not
+license_dist(const gui_dist_t *dist)	// I - Distribution to license
+{
+  char		licfile[1024];		// License filename
+  struct stat	licinfo;		// License file info
+  static int	liclength = 0;		// Size of license file
+  static char	liclabel[1024];		// Label for license pane
+
+
+  // See if we need to show the license file...
+  sprintf(licfile, "%s.license", dist->product);
+  if (!stat(licfile, &licinfo) && licinfo.st_size != liclength)
+  {
+    // Save this license's length...
+    liclength = licinfo.st_size;
+
+    // Set the title string...
+    snprintf(liclabel, sizeof(liclabel), "Software License for %s:", dist->name);
+    LicenseFile->label(liclabel);
+
+    // Load the license into the viewer...
+    LicenseFile->textfont(FL_HELVETICA);
+    LicenseFile->textsize(14);
+
+    gui_load_file(LicenseFile, licfile);
+
+    // Show the license window and wait for the user...
+    Pane[PANE_LICENSE]->show();
+    Title[PANE_LICENSE]->activate();
+    LicenseAccept->clear();
+    LicenseDecline->clear();
+    NextButton->deactivate();
+    CancelButton->activate();
+
+    while (Pane[PANE_LICENSE]->visible())
+      Fl::wait();
+
+    Title[PANE_INSTALL]->deactivate();
+
+    CancelButton->deactivate();
+    NextButton->deactivate();
+
+    if (LicenseDecline->value())
+    {
+      // Can't install without acceptance...
+      char	message[1024];		// Message for log
+
+
+      liclength = 0;
+      snprintf(message, sizeof(message), "License not accepted for %s!",
+               dist->name);
+      InstallLog->add(message);
+      return (1);
+    }
+  }
+
+  return (0);
+}
+
+
 //
 // 'list_cb()' - Handle selections in the software list.
 //
@@ -525,9 +617,9 @@ void
 list_cb(Fl_Check_Browser *, void *)
 {
   int		i, j, k;
-  dist_t	*dist,
+  gui_dist_t	*dist,
 		*dist2;
-  depend_t	*depend;
+  gui_depend_t	*depend;
 
 
   if (SoftwareList->nchecked() == 0)
@@ -546,7 +638,8 @@ list_cb(Fl_Check_Browser *, void *)
         switch (depend->type)
 	{
 	  case DEPEND_REQUIRES :
-	      if ((dist2 = find_dist(depend->product, NumDists, Dists)) != NULL)
+	      if ((dist2 = gui_find_dist(depend->product, NumDists,
+	                                 Dists)) != NULL)
 	      {
   		// Software is in the list, is it selected?
 	        k = dist2 - Dists;
@@ -563,8 +656,8 @@ list_cb(Fl_Check_Browser *, void *)
 		  break;
 		}
 	      }
-	      else if ((dist2 = find_dist(depend->product, NumInstalled,
-	                                  Installed)) == NULL)
+	      else if ((dist2 = gui_find_dist(depend->product, NumInstalled,
+	                                      Installed)) == NULL)
 	      {
 		// Required but not installed or available!
 		fl_alert("%s requires %s to be installed, but it is not available "
@@ -575,8 +668,8 @@ list_cb(Fl_Check_Browser *, void *)
 	      break;
 
           case DEPEND_INCOMPAT :
-	      if ((dist2 = find_dist(depend->product, NumInstalled,
-	                             Installed)) != NULL)
+	      if ((dist2 = gui_find_dist(depend->product, NumInstalled,
+	                                 Installed)) != NULL)
 	      {
 		// Already installed!
 		fl_alert("%s is incompatible with %s. Please remove it before "
@@ -584,7 +677,8 @@ list_cb(Fl_Check_Browser *, void *)
 		SoftwareList->checked(i + 1, 0);
 		break;
 	      }
-	      else if ((dist2 = find_dist(depend->product, NumDists, Dists)) != NULL)
+	      else if ((dist2 = gui_find_dist(depend->product, NumDists,
+	                                      Dists)) != NULL)
 	      {
   		// Software is in the list, is it selected?
 	        k = dist2 - Dists;
@@ -635,6 +729,49 @@ load_image(void)
 }
 
 
+//
+// 'load_readme()' - Load the readme file...
+//
+
+void
+load_readme(void)
+{
+  ReadmeFile->textfont(FL_HELVETICA);
+  ReadmeFile->textsize(14);
+
+  if (access("setup.readme", 0))
+  {
+    int		i;			// Looping var
+    gui_dist_t	*dist;			// Current distribution
+    char	*buffer,		// Text buffer
+		*ptr;			// Pointer into buffer
+
+
+    buffer = new char[1024 + NumDists * 300];
+
+    strcpy(buffer,
+           "This program allows you to install the following software:\n"
+	   "<ul>\n");
+    ptr = buffer + strlen(buffer);
+
+    for (i = NumDists, dist = Dists; i > 0; i --, dist ++)
+    {
+      sprintf(ptr, "<li>%s, %s\n", dist->name, dist->version);
+      ptr += strlen(ptr);
+    }
+
+    strcpy(ptr, "</ul>");
+
+    ReadmeFile->value(buffer);
+
+    delete[] buffer;
+  }
+  else
+    gui_load_file(ReadmeFile, "setup.readme");
+ 
+}
+
+
 //
 // 'load_types()' - Load the installation types from the setup.types file.
 //
@@ -647,8 +784,8 @@ load_types(void)
   char		line[1024],	// Line from file
 		*lineptr,	// Pointer into line
 		*sep;		// Separator
-  dtype_t	*dt;		// Current install type
-  dist_t	*dist;		// Distribution
+  gui_intype_t	*dt;		// Current install type
+  gui_dist_t	*dist;		// Distribution
 
 
   NumInstTypes = 0;
@@ -684,7 +821,7 @@ load_types(void)
         dt = InstTypes + NumInstTypes;
 	NumInstTypes ++;
 
-	memset(dt, 0, sizeof(dtype_t));
+	memset(dt, 0, sizeof(gui_intype_t));
 	if ((sep = strchr(lineptr, '/')) != NULL)
 	  *sep++ = '\0';
 	else
@@ -708,13 +845,13 @@ load_types(void)
         for (lineptr = line + 8; isspace(*lineptr); lineptr ++);
 
 	// Add product to list...
-        if ((dist = find_dist(lineptr, NumDists, Dists)) != NULL)
+        if ((dist = gui_find_dist(lineptr, NumDists, Dists)) != NULL)
 	{
           dt->products[dt->num_products] = dist - Dists;
 	  dt->num_products ++;
 	  dt->size += dist->rootsize + dist->usrsize;
 
-          if ((dist = find_dist(lineptr, NumInstalled, Installed)) != NULL)
+          if ((dist = gui_find_dist(lineptr, NumInstalled, Installed)) != NULL)
 	    dt->size -= dist->rootsize + dist->usrsize;
 	}
 	else
@@ -731,6 +868,14 @@ load_types(void)
 
     fclose(fp);
   }
+  else
+  {
+    Title[PANE_TYPE]->hide();
+    Title[PANE_SELECT]->position(10, 35);
+    Title[PANE_CONFIRM]->position(10, 60);
+    Title[PANE_LICENSE]->position(10, 85);
+    Title[PANE_INSTALL]->position(10, 110);
+  }
 
   for (i = 0, dt = InstTypes; i < NumInstTypes; i ++, dt ++)
   {
@@ -823,12 +968,12 @@ next_cb(Fl_Button *, void *)
 
   PrevButton->deactivate();
 
-  if (Wizard->value() == TypePane)
+  if (Wizard->value() == Pane[PANE_TYPE])
   {
     if (NumInstTypes == 0)
       Wizard->next();
   }
-  else if (Wizard->value() == SoftwarePane)
+  else if (Wizard->value() == Pane[PANE_SELECT])
   {
     if (NumInstTypes)
       PrevButton->activate();
@@ -851,7 +996,7 @@ next_cb(Fl_Button *, void *)
     }
   }
 
-  if (Wizard->value() == ConfirmPane)
+  if (Wizard->value() == Pane[PANE_CONFIRM])
   {
     ConfirmList->clear();
     PrevButton->activate();
@@ -860,13 +1005,28 @@ next_cb(Fl_Button *, void *)
       if (SoftwareList->checked(i + 1))
         ConfirmList->add(SoftwareList->text(i + 1));
   }
-  else if (Wizard->value() == LicensePane)
+  else if (Wizard->value() == Pane[PANE_LICENSE])
     Wizard->next();
 
-  if (Wizard->value() == InstallPane && !installing)
+  if (Wizard->value() == Pane[PANE_INSTALL] && !installing)
   {
+    // Show the licenses for each of the selected software packages...
     installing = 1;
 
+    for (i = 0, progress = 0, error = 0; i < NumDists; i ++)
+      if (SoftwareList->checked(i + 1) && license_dist(Dists + i))
+      {
+        InstallPercent->label("Installation Canceled!");
+	Pane[PANE_INSTALL]->redraw();
+
+        CancelButton->label("Close");
+	CancelButton->activate();
+
+	fl_beep();
+	return;
+      }
+
+    // Then do the installs...
     NextButton->deactivate();
     CancelButton->deactivate();
     CancelButton->label("Close");
@@ -879,7 +1039,7 @@ next_cb(Fl_Button *, void *)
 
         InstallPercent->value(100.0 * progress / SoftwareList->nchecked());
 	InstallPercent->label(message);
-	InstallPane->redraw();
+	Pane[PANE_INSTALL]->redraw();
 
         if ((error = install_dist(Dists + i)) != 0)
 	  break;
@@ -894,7 +1054,7 @@ next_cb(Fl_Button *, void *)
     else
       InstallPercent->label("Installation Complete");
 
-    InstallPane->redraw();
+    Pane[PANE_INSTALL]->redraw();
 
     CancelButton->activate();
 
@@ -902,9 +1062,17 @@ next_cb(Fl_Button *, void *)
 
     installing = 0;
   }
-  else if (Wizard->value() == SoftwarePane &&
+  else if (Wizard->value() == Pane[PANE_SELECT] &&
            SoftwareList->nchecked() == 0)
     NextButton->deactivate();
+
+  for (i = 0; i <= PANE_INSTALL; i ++)
+  {
+    Title[i]->activate();
+
+    if (Pane[i]->visible())
+      break;
+  }
 }
 
 
@@ -916,8 +1084,8 @@ void
 type_cb(Fl_Round_Button *w, void *)	// I - Radio button widget
 {
   int		i;		// Looping var
-  dtype_t	*dt;		// Current install type
-  dist_t	*temp,		// Current software
+  gui_intype_t	*dt;		// Current install type
+  gui_dist_t	*temp,		// Current software
 		*installed;	// Installed software
 
 
@@ -939,7 +1107,8 @@ type_cb(Fl_Round_Button *w, void *)	// I - Radio button widget
   // And then any upgrade products...
   for (i = 0, temp = Dists; i < NumDists; i ++, temp ++)
   {
-    if ((installed = find_dist(temp->product, NumInstalled, Installed)) != NULL &&
+    if ((installed = gui_find_dist(temp->product, NumInstalled,
+                                   Installed)) != NULL &&
         installed->vernumber < temp->vernumber)
       SoftwareList->checked(i + 1, 1);
   }
@@ -958,7 +1127,7 @@ void
 update_sizes(void)
 {
   int		i;		// Looping var
-  dist_t	*dist,		// Distribution
+  gui_dist_t	*dist,		// Distribution
 		*installed;	// Installed distribution
   int		rootsize,	// Total root size difference in kbytes
 		usrsize;	// Total /usr size difference in kbytes
@@ -978,8 +1147,8 @@ update_sizes(void)
       rootsize += dist->rootsize;
       usrsize  += dist->usrsize;
 
-      if ((installed = find_dist(dist->product, NumInstalled,
-                                 Installed)) != NULL)
+      if ((installed = gui_find_dist(dist->product, NumInstalled,
+                                     Installed)) != NULL)
       {
         rootsize -= installed->rootsize;
 	usrsize  -= installed->usrsize;
@@ -1047,5 +1216,5 @@ update_sizes(void)
 
 
 //
-// End of "$Id: setup2.cxx 645 2005-10-14 18:01:51Z mike $".
+// End of "$Id: setup2.cxx 739 2006-12-05 17:20:27Z mike $".
 //
