@@ -1,9 +1,9 @@
 //
-// "$Id: uninst2.cxx 645 2005-10-14 18:01:51Z mike $"
+// "$Id: uninst2.cxx 739 2006-12-05 17:20:27Z mike $"
 //
 //   ESP Software Removal Wizard main entry for the ESP Package Manager (EPM).
 //
-//   Copyright 1999-2005 by Easy Software Products.
+//   Copyright 1999-2006 by Easy Software Products.
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 //   main()           - Main entry for software wizard...
 //   list_cb()        - Handle selections in the software list.
 //   load_image()     - Load the setup image file (setup.gif/xpm)...
+//   load_readme()    - Load the readme file...
 //   log_cb()         - Add one or more lines of text to the removal log.
 //   next_cb()        - Show software selections or remove software.
 //   remove_dist()    - Remove a distribution...
@@ -73,13 +74,24 @@ AuthorizationRef SetupAuthorizationRef;
 #endif // __APPLE__
 
 
+//
+// Panes...
+//
+
+#define PANE_WELCOME	0
+#define PANE_SELECT	1
+#define PANE_CONFIRM	2
+#define PANE_REMOVE	3
+
+
 //
 // Local functions...
 //
 
 void	load_image(void);
+void	load_readme(void);
 void	log_cb(int fd, int *fdptr);
-int	remove_dist(const dist_t *dist);
+int	remove_dist(const gui_dist_t *dist);
 void	show_installed(void);
 void	update_sizes(void);
 
@@ -95,10 +107,9 @@ main(int  argc,			// I - Number of command-line arguments
   Fl_Window	*w;		// Main window...
 
 
-#if !defined(__hpux) && !defined(__sun) && !defined(__osf) && !defined(_AIX)
-  // Use modern "skin" for modern OS's...
-  Fl::scheme("plastic");
-#endif // !__hpux && !__sun && !__osf && !_AIX
+  // Use GTK+ scheme for all operating systems...
+  Fl::background(230, 230, 230);
+  Fl::scheme("gtk+");
 
 #ifdef __APPLE__
   // OSX passes an extra command-line option when run from the Finder.
@@ -124,20 +135,21 @@ main(int  argc,			// I - Number of command-line arguments
       }
     }
   }
-#else
-  // Use the default scheme on this system...
-  Fl::scheme(NULL);
 #endif // __APPLE__
 
   w = make_window();
 
-  WelcomePane->show();
+  Pane[PANE_WELCOME]->show();
   PrevButton->deactivate();
   NextButton->deactivate();
 
-  load_image();
+  gui_get_installed();
+  show_installed();
 
-  w->show(1, argv);
+  load_image();
+  load_readme();
+
+  w->show();
 
   while (!w->visible())
     Fl::wait();
@@ -176,9 +188,6 @@ main(int  argc,			// I - Number of command-line arguments
   }
 #endif // __APPLE__
 
-  get_installed();
-  show_installed();
-
   NextButton->activate();
 
   Fl::run();
@@ -199,9 +208,9 @@ void
 list_cb(Fl_Check_Browser *, void *)
 {
   int		i, j, k;
-  dist_t	*dist,
+  gui_dist_t	*dist,
 		*dist2;
-  depend_t	*depend;
+  gui_depend_t	*depend;
 
 
   if (SoftwareList->nchecked() == 0)
@@ -220,7 +229,8 @@ list_cb(Fl_Check_Browser *, void *)
         switch (depend->type)
 	{
 	  case DEPEND_REQUIRES :
-	      if ((dist2 = find_dist(depend->product, NumInstalled, Installed)) != NULL)
+	      if ((dist2 = gui_find_dist(depend->product, NumInstalled,
+	                                 Installed)) != NULL)
 	      {
   		// Software is in the list, is it selected?
 	        k = dist2 - Installed;
@@ -237,8 +247,8 @@ list_cb(Fl_Check_Browser *, void *)
 		  break;
 		}
 	      }
-	      else if ((dist2 = find_dist(depend->product, NumInstalled,
-	                                  Installed)) == NULL)
+	      else if ((dist2 = gui_find_dist(depend->product, NumInstalled,
+	                                      Installed)) == NULL)
 	      {
 		// Required but not installed or available!
 		fl_alert("%s requires %s to be installed, but it is not available "
@@ -249,8 +259,8 @@ list_cb(Fl_Check_Browser *, void *)
 	      break;
 
           case DEPEND_INCOMPAT :
-	      if ((dist2 = find_dist(depend->product, NumInstalled,
-	                             Installed)) != NULL)
+	      if ((dist2 = gui_find_dist(depend->product, NumInstalled,
+	                                 Installed)) != NULL)
 	      {
 		// Already installed!
 		fl_alert("%s is incompatible with %s. Please remove it before "
@@ -258,7 +268,8 @@ list_cb(Fl_Check_Browser *, void *)
 		SoftwareList->checked(i + 1, 0);
 		break;
 	      }
-	      else if ((dist2 = find_dist(depend->product, NumInstalled, Installed)) != NULL)
+	      else if ((dist2 = gui_find_dist(depend->product, NumInstalled,
+	                                      Installed)) != NULL)
 	      {
   		// Software is in the list, is it selected?
 	        k = dist2 - Installed;
@@ -309,6 +320,49 @@ load_image(void)
 }
 
 
+//
+// 'load_readme()' - Load the readme file...
+//
+
+void
+load_readme(void)
+{
+  ReadmeFile->textfont(FL_HELVETICA);
+  ReadmeFile->textsize(14);
+
+  if (access("uninst.readme", 0))
+  {
+    int		i;			// Looping var
+    gui_dist_t	*dist;			// Current distribution
+    char	*buffer,		// Text buffer
+		*ptr;			// Pointer into buffer
+
+
+    buffer = new char[1024 + NumInstalled * 400];
+
+    strcpy(buffer, "This program allows you to remove the following "
+                   "software:</p>\n"
+		   "<ul>\n");
+    ptr = buffer + strlen(buffer);
+
+    for (i = NumInstalled, dist = Installed; i > 0; i --, dist ++)
+    {
+      sprintf(ptr, "<li>%s, %s\n", dist->name, dist->version);
+      ptr += strlen(ptr);
+    }
+
+    strcpy(ptr, "</ul>");
+
+    ReadmeFile->value(buffer);
+
+    delete[] buffer;
+  }
+  else
+    gui_load_file(ReadmeFile, "uninst.readme");
+
+}
+
+
 //
 // 'log_cb()' - Add one or more lines of text to the removal log.
 //
@@ -378,7 +432,7 @@ next_cb(Fl_Button *, void *)
 
   PrevButton->deactivate();
 
-  if (Wizard->value() == ConfirmPane)
+  if (Wizard->value() == Pane[PANE_CONFIRM])
   {
     ConfirmList->clear();
     PrevButton->activate();
@@ -388,7 +442,7 @@ next_cb(Fl_Button *, void *)
         ConfirmList->add(SoftwareList->text(i + 1));
   }
 
-  if (Wizard->value() == RemovePane && !removing)
+  if (Wizard->value() == Pane[PANE_REMOVE] && !removing)
   {
     removing = 1;
 
@@ -404,7 +458,7 @@ next_cb(Fl_Button *, void *)
 
         RemovePercent->value(100.0 * progress / SoftwareList->nchecked());
 	RemovePercent->label(message);
-	RemovePane->redraw();
+	Pane[PANE_REMOVE]->redraw();
 
         if ((error = remove_dist(Installed + i)) != 0)
 	  break;
@@ -419,7 +473,7 @@ next_cb(Fl_Button *, void *)
     else
       RemovePercent->label("Removal Complete");
 
-    RemovePane->redraw();
+    Pane[PANE_REMOVE]->redraw();
 
     CancelButton->activate();
 
@@ -427,7 +481,7 @@ next_cb(Fl_Button *, void *)
 
     removing = 0;
   }
-  else if (Wizard->value() == SoftwarePane &&
+  else if (Wizard->value() == Pane[PANE_SELECT] &&
            SoftwareList->nchecked() == 0)
     NextButton->deactivate();
 }
@@ -438,7 +492,7 @@ next_cb(Fl_Button *, void *)
 //
 
 int					// O - Remove status
-remove_dist(const dist_t *dist)		// I - Distribution to remove
+remove_dist(const gui_dist_t *dist)	// I - Distribution to remove
 {
   char		command[1024];		// Command string
   int		fds[2];			// Pipe FDs
@@ -450,7 +504,9 @@ remove_dist(const dist_t *dist)		// I - Distribution to remove
   snprintf(command, sizeof(command), "**** %s ****", dist->name);
   RemoveLog->add(command);
 
-  snprintf(command, sizeof(command), EPM_SOFTWARE "/%s.remove", dist->product);
+  if (dist->type == PACKAGE_PORTABLE)
+    snprintf(command, sizeof(command), EPM_SOFTWARE "/%s.remove",
+             dist->product);
 
 #ifdef __APPLE__
   // Run the remove script using Apple's authorization API...
@@ -486,7 +542,11 @@ remove_dist(const dist_t *dist)		// I - Distribution to remove
     close(fds[1]);
 
     // Execute the command; if an error occurs, return it...
-    execl(command, command, "now", NULL);
+    if (dist->type == PACKAGE_PORTABLE)
+      execl(command, command, "now", (char *)0);
+    else
+      execlp("rpm", "rpm", "-e", "--nodeps", dist->product, (char *)0);
+
     exit(errno);
   }
   else if (pid < 0)
@@ -559,7 +619,7 @@ void
 show_installed()
 {
   int		i;		// Looping var
-  dist_t	*temp;		// Pointer to current distribution
+  gui_dist_t	*temp;		// Pointer to current distribution
   char		line[1024];	// Product name and version...
 
 
@@ -588,7 +648,7 @@ void
 update_sizes(void)
 {
   int		i;		// Looping var
-  dist_t	*dist,		// Distribution
+  gui_dist_t	*dist,		// Distribution
 		*installed;	// Installed distribution
   int		rootsize,	// Total root size difference in kbytes
 		usrsize;	// Total /usr size difference in kbytes
@@ -608,8 +668,8 @@ update_sizes(void)
       rootsize += dist->rootsize;
       usrsize  += dist->usrsize;
 
-      if ((installed = find_dist(dist->product, NumInstalled,
-                                 Installed)) != NULL)
+      if ((installed = gui_find_dist(dist->product, NumInstalled,
+                                     Installed)) != NULL)
       {
         rootsize -= installed->rootsize;
 	usrsize  -= installed->usrsize;
@@ -677,5 +737,5 @@ update_sizes(void)
 
 
 //
-// End of "$Id: uninst2.cxx 645 2005-10-14 18:01:51Z mike $".
+// End of "$Id: uninst2.cxx 739 2006-12-05 17:20:27Z mike $".
 //
